 
var ffi = require('ffi');
var fs = require('fs');
var path = require('path');

var libm = ffi.Library(`${__dirname}/bin/${process.arch}/codeflex.dll`, {
    'loadAssemblies':['void',[]],
    'compileCode': ['String',['String']],
    'createInstance':['String',['String','String']],
    'executeMethod':['String',['String','String','String','String']],
});

function reverseString(str) {
    var newString = "";
    for (var i = str.length - 1; i >= 0; i--) {
        newString += str[i];
    }
    return newString;
};

try{
    libm.loadAssemblies();
}catch(ex){
    console.log(ex);
}

module.exports = {
    compileCode:function(language, statement, callBack){
        language = language || "vb";

        statement = statement.toString();
        statement = statement.substring(statement.indexOf("*")+1);
        statement = reverseString(statement);
        statement = statement.substring(statement.indexOf("*")+1);
        statement = reverseString(statement);
     
        try{
            var config ={
                language:language,
                code:[
                    statement
                ]
            };
            libm.compileCode.async(JSON.stringify(config), function (err, res) {
                callBack(err,res);
            });
        }catch(ex){
            callBack(ex,null);
        }
    },
    compileDirectory:function(language, directory, extension, callBack){
        language = language || "vb";

        var config ={
            language:language,
            code:[]
        };

        var compile = function(){
            try{
                libm.compileCode.async(JSON.stringify(config), function (err, res) {
                    callBack(err,res);
                });
            }catch(ex){
                callBack(ex,null);
            }
        };

        function getFiles (dir, files_){
            files_ = files_ || [];
            var files = fs.readdirSync(dir);
            for (var i in files){
                var name = path.join(dir, files[i]);
                if (fs.statSync(name).isDirectory()){
                    getFiles(name, files_);
                } else {
                    if(path.extname(name).toString().toLowerCase()==extension.toString().toLowerCase()){
                        files_.push(name);
                    }
                }
            }
            return files_;
        };
 
        files = getFiles(directory);
      
        var nextFile = function(){
            var filename = files[0];
            if(filename==null){
                compile();
                return;
            }

            files.splice(0,1);

            fs.readFile(filename, 'utf8', function(err, statement) {
                if(err){
                    callBack(err,null);
                    return;
                }
                config.code.push(statement);
                nextFile();
            });
        };

        nextFile();
    },
    createInstance:function(assemblyId, instanceType, callBack){
        try{
            libm.createInstance.async(assemblyId, instanceType,function (err, res) {
                callBack(err,res);
            });
        }catch(ex){
            callBack(ex,null);
        }
    },
    executeMethod(assemblyId, instanceId, method, params, callBack){
        try{
            params = JSON.stringify(params);
            libm.executeMethod.async(assemblyId, instanceId,method,params,function (err, res) {
                callBack(err,res);
            });
        }catch(ex){
            callBack(ex,null);
        }
    }
};